import uvicorn

from loguru import logger

from .__version__ import __version__
from .config import config


def main():
    logger.info(f'plyder v{__version__}')

    uvicorn.run(
        'plyder.app:app',
        host=config['ip_host'],
        port=config['port'],
        log_level='warning',
        reload=False,
    )


if __name__ == '__main__':
    main()



import importlib.resources as pkg_resources

from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles

from loguru import logger

from . import static
from .routes import server

from .config import config
from .downloader import clean_packages


app = FastAPI()

with pkg_resources.path(static, 'styles.css') as static_file:
    app.mount('/static', StaticFiles(directory=static_file.parent), name='static')

app.include_router(server.router)


@app.on_event('startup')
def startup_event():
    config['download_directory'].mkdir(parents=True, exist_ok=True)
    clean_packages()
    logger.info(f'Server {config["ip_host"]}:{config["port"]} started')



import sys
import json
from urllib.parse import urlparse
from contextlib import redirect_stdout

# from mega import Mega
import sh

from loguru import logger

from .config import config


STATUS_FILENAME = '.plyder.status'
LOG_FILENAME = '.download.log'


def download_mega(url: str, output_dir: str):
    # mega = Mega()
    # m = mega.login()
    # m.download_url(url=url, dest_path=output_dir)

    sh.megadl(
        '--path',
        output_dir,
        url,
        _out=sys.stdout,
        _err_to_out=True,
    )


def download_wget(url: str, output_dir: str):
    sh.wget(
        '--directory-prefix',
        output_dir,
        url,
        _out=sys.stdout,
        _err_to_out=True,
    )


PROVIDER_DICT = {'mega.nz': download_mega}


@logger.catch
def download_url(url: str, output_dir: str):
    o = urlparse(url)

    provider = PROVIDER_DICT.get(o.netloc)
    if provider is None:
        logger.warning(f'No provider for "{url}" found, using wget fallback')
        provider = download_wget

    logger.info(f'[{provider.__name__}] Downloading "{url}" to "{output_dir}"')

    try:
        provider(url, output_dir)
    except Exception as e:
        logger.exception(e)
        return False
    return True


def download_package(job: 'JobSubmission'):
    output_dir = config['download_directory'] / job.package_name
    output_dir.mkdir(parents=True, exist_ok=True)

    logger.info(f'Processing "{job.package_name}"')
    with (output_dir / STATUS_FILENAME).open('w') as fd:
        json.dump({'status': 'running'}, fd)

    with (output_dir / LOG_FILENAME).open('w') as fd:
        with redirect_stdout(fd):
            any_url_failed = False
            for url in job.url_field:
                success = download_url(url, output_dir)
                any_url_failed |= not success

    logger.info(f'Finished "{job.package_name}"')
    with (output_dir / STATUS_FILENAME).open('w') as fd:
        json.dump({'status': 'failed' if any_url_failed else 'done'}, fd)


def clean_packages():
    if not config['download_directory'].exists():
        logger.warning(
            f'Download directory ({config["download_directory"]}) does not exist.'
        )
        return

    for entry in config['download_directory'].iterdir():
        if not entry.is_dir():
            continue

        status_file = entry / STATUS_FILENAME
        if not status_file.exists():
            continue

        with status_file.open() as fd:
            info = json.load(fd)

        if info['status'] not in ('done', 'failed'):
            logger.warning(
                f'Package "{entry.name}" in inconsistent state, setting to failed'
            )

            with status_file.open('w') as fd:
                json.dump({'status': 'failed'}, fd)


def list_packages():
    if not config['download_directory'].exists():
        logger.warning(
            f'Download directory ({config["download_directory"]}) does not exist.'
        )
        return []

    res = []
    for entry in config['download_directory'].iterdir():
        # read log
        log_file = entry / LOG_FILENAME
        if log_file.exists():
            with log_file.open() as fd:
                log_text = fd.read()[-10000:]  # truncate
        else:
            log_text = ''

        # assemble information
        status_file = entry / STATUS_FILENAME
        if status_file.exists():
            with status_file.open() as fd:
                info = json.load(fd)
        else:
            info = {'status': 'unknown'}

        res.append({'name': entry.name, 'info': info, 'log': log_text})
    return res



import importlib.resources as pkg_resources

from fastapi import APIRouter, BackgroundTasks, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates

from pydantic import BaseModel, validator

from ..__version__ import __version__
from .. import templates
from ..downloader import download_package, list_packages


router = APIRouter(tags=['server'])

with pkg_resources.path(templates, 'index.html') as template_file:
    templates = Jinja2Templates(directory=template_file.parent)


class JobSubmission(BaseModel):
    package_name: str
    url_field: str

    @validator('package_name', 'url_field')
    def is_nonempty(cls, value):
        if not value:
            raise ValueError('must be non-empty')
        return value

    @validator('url_field')
    def create_url_list(cls, value):
        return [url.strip() for url in value.splitlines() if url.strip()]


@router.get('/', response_class=HTMLResponse)
async def root(request: Request):
    return templates.TemplateResponse(
        'index.html',
        {'request': request, 'version': __version__, 'package_list': list_packages()},
    )


@router.post('/submit_job')
async def submit_job(background_tasks: BackgroundTasks, job: JobSubmission):
    background_tasks.add_task(download_package, job=job)
    return {'status': 'good'}



from pathlib import Path

import yaml
import jsonschema
from appdirs import user_config_dir

from loguru import logger


DEFAULT_CONFIG = {
    'download_directory': '~/Downloads/',
    'ip_host': '0.0.0.0',
    'port': 5000,
}

CONFIG_SCHEMA = {
    '$schema': 'http://json-schema.org/draft-07/schema#',
    'properties': {
        'download_directory': {'type': 'string'},
        'ip_host': {'type': 'string'},
        'port': {'type': 'integer'},
    },
    'additionalProperties': False,
    'required': list(DEFAULT_CONFIG.keys()),
}


def get_config():
    config_file = Path(user_config_dir('plyder')) / 'config.yaml'

    # load config
    if config_file.exists():
        with config_file.open() as fd:
            config = yaml.safe_load(fd)
    else:
        logger.warning(
            'Could not find configuration file. '
            f'Create "{config_file}" to set your own options. '
            f'Using the default config for now: {DEFAULT_CONFIG}'
        )
        config = DEFAULT_CONFIG

    # validate config
    jsonschema.validate(instance=config, schema=CONFIG_SCHEMA)

    # prepare some values
    config['download_directory'] = Path(config['download_directory']).expanduser()

    return config


config = get_config()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['plyder', 'plyder.routes', 'plyder.static', 'plyder.templates']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'aiofiles>=0.6.0,<0.7.0',
 'appdirs>=1.4.4,<2.0.0',
 'fastapi>=0.63.0,<0.64.0',
 'jsonschema>=3.2.0,<4.0.0',
 'loguru>=0.5.3,<0.6.0',
 'mega.py>=1.0.8,<2.0.0',
 'sh>=1.14.1,<2.0.0',
 'uvicorn>=0.13.4,<0.14.0']

entry_points = \
{'console_scripts': ['plyder = plyder:main']}

setup_kwargs = {
    'name': 'plyder',
    'version': '0.2.1',
    'description': 'Download manager with web-interface.',
    'long_description': '# plyder\n\n[![PyPI](https://img.shields.io/pypi/v/plyder.svg?style=flat)](https://pypi.python.org/pypi/plyder)\n[![Tests](https://github.com/kpj/plyder/workflows/Tests/badge.svg)](https://github.com/kpj/plyder/actions)\n\nDownload manager with web-interface.\n\n\n## Installation\n\n```python\n$ pip install plyder\n```\n\n\n## Usage\n\n```bash\n$ plyder\n```\n\n`plyder` works out of the box. Though you might want to adapt the configuration to your taste.\n',
    'author': 'kpj',
    'author_email': 'kim.philipp.jablonski@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kpj/plyder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)
